// -*- mode: C++; c-file-style: "Stroustrup" -*-
//
// This file is part of krot,
// a program for the simulation, assignment and fit of HRLIF spectra.
//
// Copyright (C) 1998 Jochen Kpper
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; see the file License. if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
//
// If you use this program for your scientific work, please cite it according to
// the file CITATION that comes with the package.
//
// $Id: scrollbar64.h,v 1.2 1998/09/28 13:55:27 jochen Exp $



#ifndef KROT_SCROLLBAR_H
#define KROT_SCROLLBAR_H


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <qglobal.h>
#include <qscrollbar.h>
#include <qwidget.h>

#include <sys/types.h>



#if ( SIZEOF_INT >= 8 )


/**
 * SrcollBar for KRot.
 *
 * If sizeof( int ) >= sizeof( int64_t ) Qt implements all we need.
 */
typedef QScrollBar ScrollBar64;


#endif
#if ( SIZEOF_INT < 8 )
// #else // SIZEOF_INT  moc from ( QT_VERSION == 140 ) is causing trouble with conditional compilation !




/**
 * @short SrcollBar for KRot.
 * @author Jochen Kpper
 * @version $Id: scrollbar64.h,v 1.2 1998/09/28 13:55:27 jochen Exp $
 *
 * This is an enhancement of the standard Qt Scrollbar to use int64_t parameter
 * in all functions for ranges and values.
 *
 * Due to accesibility restrictions this class is actually not derived from
 * @ref QScrollBar but does have a member of type @ref QScrollBar that does the
 * real work.
 * Here we perform translations from int64_t to int where needed and call the
 * approbiate @ref QScrollBar member functions.
 *
 * Only needed functionality is implemented. If you need more, feel free to
 * extend this class. But keep <b>exactly</b> the @ref QScrollBar interface !
 * If you need more/other functionality, subclass this class and do it there.
 *
 * @author Jochen Kpper
 * @version $Id: scrollbar64.h,v 1.2 1998/09/28 13:55:27 jochen Exp $
 */
class ScrollBar64 : public QWidget
{
    Q_OBJECT

public:

    /**
     * Possible orientations of the ScrollBar. To keep the QScrollBar interface,
     * we define the type to be the same as in QScrollBar and the values to
     * match the QScrollBar ones.
     */
    typedef QScrollBar::Orientation Orientation;
    static const Orientation Horizontal = QScrollBar::Horizontal;
    static const Orientation Vertical = QScrollBar::Vertical;

    /**
     * Initialize data and call the corresponding QScrollBar constructor.
     */
    ScrollBar64( QWidget *parent=0, const char *name=0 );

    /**
     * Initialize data and call the corresponding QScrollBar constructor.
     */
    ScrollBar64( Orientation orient, QWidget *parent=0, const char *name=0 );

    /**
     * Initialize data, do necessary transformations and call the corresponding
     * QScrollBar constructor with approbiate (translated) parameters.
     */
    ScrollBar64( int64_t minValue, int64_t maxValue, int64_t lineStep,
	       int64_t pageStep, int64_t value, Orientation orient,
	       QWidget *parent=0, const char *name=0 );

    /**
     * Destructor.
     */
    virtual ~ScrollBar64();

    /**
     * Dragging status.
     * @return True if the user has clicked the mouse on the slider and is
     *         currenly dragging it, false if not. 
     */
    bool draggingSlider() const;

    /**
     * Returns the scroll bar orientation which might be ScrollBar::Horizontal
     * or ScrollBar::Vertical.
     *
     * See also: @ref setOrientation.
     */
    Orientation orientation() const;

    /**
     * Sets the scroll bar orientation.
     * The orientation must be ScrollBar::Horizontal or ScrollBar::Vertical.
     *
     * See also: @ref orientation.
     */
    void setOrientation( Orientation );

    /**
     * Reimplements the virtual function @ref QWidget::setPalette. 
     *
     * Sets the background color to the mid color for Motif style scroll bars. 
     *
     * Reimplemented from @ref QWidget.
     */
    virtual void setPalette( const QPalette & );

    /**
     * Set ScrollBar range.
     *
     * @param min Lower bound.
     * @param max Upper bound.
     */
    void setRange( int64_t min, int64_t max );

    /**
     * Sets the ScrollBar line step and page step.
     *
     * @param line New line step size.
     * @param page New page step size.
     *
     * See also: @ref setRange ().
     */
    void setSteps( int64_t line, int64_t page );

    /**    
     * Enables ScrollBar tracking if enable is true, or disables tracking if
     * enable is false.
     *
     * If tracking is enabled (default), the scroll bar emits the
     * @ref valueChanged signal whenever the slider is being dragged. If
     * tracking is disabled, the ScrollBar emits the @ref valueChanged signal
     * when the user relases the mouse button (unless the value happens to be
     * the same as before).
     *
     * See also: @ref tracking. 
     */
    void setTracking( bool enable );

    /**
     * Set ScrollBar value.
     *
     * @param value The value to set at.
     */
    void setValue( int64_t value );

    /**
     * Reimplemented from @ref QWidget.
     *
     * @return Size hint for this scroll bar.
     */
    virtual QSize sizeHint() const;

    /**
     * Returns true if tracking is enabled, or false if tracking is disabled.
     *
     * Tracking is initially enabled.
     *
     *See also: @ref setTracking.
     */
    bool tracking() const;

    /**
     * Returns the current ScrollBar value. 
     *
     * See also: @ref #setValue ().
     */
    int64_t value() const;


    signals:

    /**
     * Emitted when the ScrollBar value is changed.
     *
     * @param value New ScrollBar value.
     */
    void valueChanged( int64_t value );
    
    /**
     * Emitted when the slider is moved by the user.
     *
     * @param value New ScrollBar value.
     */
    void sliderMoved( int64_t value );


protected:

    /**
     * Handle resize events.
     *
     * Keeps the height and sets the widget to its maximum width.
     */
    virtual void resizeEvent( QResizeEvent *event );

    /**
     * @internal
     * Initialization and internal connections.
     */
    void init();

    /**
     * @internal
     * Determine translation and set QScrollBar range.
     *
     * Calls QScrollBar::setRange with the transformed values.
     */
    void doIt();


private slots:

    /**
     * @internal Called when the user moved the QScrollBar slider.
     *
     * @param value New QScrollBar value.
     */
    void slotSliderMoved( int value );

    /**
     * @internal Called when the QScrollBar value is changed.
     *
     * @param value New QScrollBar value.
     */
    void slotValueChanged( int value );


protected:

    /**
     * @internal
     */
    QScrollBar *qscrollbar;

    /**
     * @internal
     */
    int64_t div, line, max, min, page, shift, val;
};



#include "scrollbar64_inline.h"



#endif // SIZEOF_INT



#endif
